<?php

namespace TVn7\DomainBundle\Entity;

use Doctrine\ORM\Mapping as ORM;
use TVn7\DomainBundle\Entity\Alias;

use Symfony\Component\Validator\Constraints as Assert;
use TVn7\DomainBundle\Validator\Constraints as TVn7Assert;
use Symfony\Component\Validator\Context\ExecutionContextInterface;
use Gedmo\Mapping\Annotation as Gedmo;

/*
 * ### TVn7DomainBundle:Target ###
 * Cette entité est utilisée par Postfix sur Galileo pour gérer les mails de TVn7.
 *
 * Champs critiques :
 *  * addr
 *  * enabled
 *  * alias (alias_id)
 *
 * Plus d'info ici : https://git.inpt.fr/inp-net/conf-mail/blob/master/tvn7.cf
 */

/**
 * Target
 *
 * @ORM\Table(name="tdb_domain_target")
 * @ORM\Entity(repositoryClass="TVn7\DomainBundle\Repository\TargetRepository")
 * @TVn7Assert\NoAliasLoop()
 * @Gedmo\Loggable(logEntryClass="TVn7\LogBundle\Entity\AppsLog")
 */
class Target
{
    /**
     * @var int
     *
     * @ORM\Column(name="id", type="integer")
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="addr", type="string", length=255)
     * @Assert\Email(
     *     message = "{{ value }} n'est pas un email valide.",
     *     checkMX = true,
     *     strict = true
     * )
     * @Gedmo\Versioned
     */
    private $addr;

    /**
     * @var bool
     *
     * @ORM\Column(name="enabled", type="boolean", options={"default": 1})
     * @Gedmo\Versioned
     */
    private $enabled = true;

    /**
     * @var Alias
     * @ORM\ManyToOne(targetEntity="TVn7\DomainBundle\Entity\Alias", inversedBy="targets")
     * @ORM\JoinColumn(name="alias_id", nullable=false)
     * @Gedmo\Versioned
     */
    private $alias;

    /**
     * Constructor
     *
     * @param string $addr
     */
    public function __construct($addr = "")
    {
        $this->setAddr($addr);
    }

    /**
     * Get id
     *
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set addr
     *
     * @param string $addr
     *
     * @return Target
     */
    public function setAddr($addr)
    {
        $this->addr = strtolower($addr);

        return $this;
    }

    /**
     * Get addr
     *
     * @return string
     */
    public function getAddr()
    {
        return $this->addr;
    }

    /**
     * Set enabled
     *
     * @param boolean $enabled
     *
     * @return Target
     */
    public function setEnabled($enabled)
    {
        $this->enabled = $enabled;

        return $this;
    }

    /**
     * Get enabled
     *
     * @return bool
     */
    public function getEnabled()
    {
        return $this->enabled;
    }

    /**
     * Set alias
     *
     * @param Alias $alias
     *
     * @return Target
     */
    public function setAlias(Alias $alias)
    {
        $this->alias = $alias;

        return $this;
    }

    public function clearAlias() {
        $this->alias = null;

        return $this;
    }

    /**
     * Get alias
     *
     * @return Alias
     */
    public function getAlias()
    {
        return $this->alias;
    }

    public function __toString()
    {
        return $this->getAddr();
    }

    /**
     * @Assert\Callback
     */
    public function isValid(ExecutionContextInterface $context, $payload) {
        if($this->getAlias() instanceof Alias) {
            $alias = $this->getAlias();
        } else {
            return;
        }
        
        $nb_occ = $alias->getTargets()->filter(function($p) {
            return $p->getAddr() == $this->getAddr();
        })->count();

        if($nb_occ > 1) {
            $context->buildViolation('Cette cible est en double')
                    ->atPath('addr')
                    ->addViolation();
        }

        /**
         * @todo Recherche récursive dans le cas où un alias TVn7 est une cible
         */
        if($this->addr == $alias->getAddr()) {
            $context->buildViolation('L\'alias ne peut pas être une de ses propres cibles.')
                    ->atPath('addr')
                    ->addViolation();
        }
    }
}
